/*
* This file is part of buteo-syncml package
*
* Copyright (C) 2010 Nokia Corporation. All rights reserved.
*
* Contact: Sateesh Kavuri <sateesh.kavuri@nokia.com>
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation 
* and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors may 
* be used to endorse or promote products derived from this software without 
* specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
* 
*/
#ifndef STORAGEPROVIDER_H
#define STORAGEPROVIDER_H

class QString;

#include "StorageContentFormatInfo.h"

namespace DataSync {

class StoragePlugin;
class SessionHandler;

/*! \brief Storage provider interface for SyncAgent
 *
 */
class StorageProvider
{
public:

    /*! \brief Constructor
     *
     */
    StorageProvider() : iSessionHandler(0) { }

    /*! \brief Destructor
     *
     */
    virtual ~StorageProvider() { }

    /*! \brief Sets session handler
     *
     * \param aSessionHandler Pointer to session handler. Pointer should be valid
     *  as long as this object is alive.
     */
    void setSessionHandler(const SessionHandler *aSessionHandler) {
        iSessionHandler = aSessionHandler;
    }

    /*! \brief Provides content format information based on storage URI
     *
     * This is used by SyncAgent to examine content information of a storage
     * plugin before acquiring it.
     *
     * @param aURI Source URI of the storage
     * @param aInfo Content information of the storage (if successful)
     * @return True if the storage was found, otherwise false
     */
    virtual bool getStorageContentFormatInfo( const QString& aURI,
                                              StorageContentFormatInfo& aInfo ) = 0;

    /*! \brief Provides a storage based on URI
     *
     * @param aURI Local URI of the storage to retrieve
     * @return Storage plugin if success, otherwise NULL
     */
    virtual StoragePlugin* acquireStorageByURI( const QString& aURI ) = 0;

    /*! \brief Provides a storage based on MIME
     *
     * @param aMIME MIME of the storage to retrieve
     * @return Storage plugin if success, otherwise NULL
     */
    virtual StoragePlugin* acquireStorageByMIME( const QString& aMIME ) = 0;

    /*! \brief Releases a storage
     *
     * @param aStorage Storage to release
     */
    virtual void releaseStorage( StoragePlugin* aStorage ) = 0;

protected:


    const SessionHandler *iSessionHandler;  /*!<Instance of the current session handler*/
};

}

#endif  //  STORAGEPROVIDER_H
