/*
* This file is part of buteo-syncml package
*
* Copyright (C) 2010 Nokia Corporation. All rights reserved.
*
* Contact: Sateesh Kavuri <sateesh.kavuri@nokia.com>
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation 
* and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors may 
* be used to endorse or promote products derived from this software without 
* specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
* 
*/

#include "SyncTargetTest.h"
#include "SyncTarget.h"
#include "DatabaseHandler.h"
#include "Mock.h"
#include "ChangeLog.h"

using namespace DataSync;

void SyncTargetTest::initTestCase()
{
    iDbHandler = new DatabaseHandler( "/tmp/synctargettest.db");
    QString remoteDevice = "remotedevice";
    QString localDb = "localcontacts";
    const SyncMode syncMode;

    iChangeLog = new ChangeLog( remoteDevice, localDb,
                                syncMode.syncDirection() );

    iStorage = new MockStorage( localDb );
    QVERIFY( iStorage != 0 );
    iSyncTarget = new SyncTarget( iChangeLog, iStorage, syncMode, "fooanchor" );
    QVERIFY( iSyncTarget != 0 );
}

void SyncTargetTest::cleanupTestCase()
{
    delete iSyncTarget;
    iSyncTarget = 0;
    delete iDbHandler;
    iDbHandler = 0;
    delete iStorage;
    iStorage = 0;
}

void SyncTargetTest::testSetGetRemoteNextAnchor()
{
    const QString next = "foo";
    iSyncTarget->setRemoteNextAnchor(next);
    QString next_received = iSyncTarget->getRemoteNextAnchor();
    QVERIFY(next == next_received);
}

void SyncTargetTest::testRevertSyncMode()
{
    iSyncTarget->revertSyncMode();
}

void SyncTargetTest::testReverted()
{
    iSyncTarget->iReverted = true;
    bool reverted = iSyncTarget->reverted();
    QVERIFY(reverted);
}

void SyncTargetTest::testClearUIDMappings()
{
    iSyncTarget->clearUIDMappings();
}

void SyncTargetTest::testSetRefreshFromClient()
{
    QCOMPARE( iSyncTarget->setRefreshFromClient(), false );
}

QTEST_MAIN(DataSync::SyncTargetTest)
