/*
* This file is part of buteo-syncml package
*
* Copyright (C) 2010 Nokia Corporation. All rights reserved.
*
* Contact: Sateesh Kavuri <sateesh.kavuri@nokia.com>
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation 
* and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors may 
* be used to endorse or promote products derived from this software without 
* specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
* 
*/
#ifndef AUTHENTICATIONPACKAGE_H
#define AUTHENTICATIONPACKAGE_H

#include "Package.h"

#include "SyncAgentConsts.h"

class AuthenticationPackageTest;

namespace DataSync {

/*! \brief Handles sync initialization phase authentication
 */
class AuthenticationPackage : public Package
{
    Q_OBJECT
public:

    /*! \brief Constructor
     *
     * @param aAuthType Type of authentication to use (either BASIC or MD5)
     * @param aUsername Username to use (BASIC and MD5)
     * @param aPassword Password to use (BASIC and MD5)
     * @param aNonce Nonce to use (MD5)
     */
    AuthenticationPackage( const AuthType& aAuthType, const QString& aUsername, const QString& aPassword, const QByteArray& aNonce = "" );

    /*! \brief Destructor
     *
     */
    virtual ~AuthenticationPackage();

    virtual bool write( SyncMLMessage& aMessage, int& aSizeThreshold, bool aWBXML, const ProtocolVersion& aVersion );

protected:

private:

    AuthType    iAuthType;
    QString     iUsername;
    QString     iPassword;
    QByteArray  iNonce;

    friend class ::AuthenticationPackageTest;
};

}

#endif  //  AUTHENTICATIONPACKAGE_H
