/*
* This file is part of buteo-syncml package
*
* Copyright (C) 2010 Nokia Corporation. All rights reserved.
*
* Contact: Sateesh Kavuri <sateesh.kavuri@nokia.com>
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation 
* and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors may 
* be used to endorse or promote products derived from this software without 
* specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
* 
*/

#ifndef SERVERCOMMANDHANDLERTEST_H
#define SERVERCOMMANDHANDLERTEST_H

#include <QTest>
#include "SyncAgentConfig.h"
#include "ResponseGenerator.h"
#include "Mock.h"

namespace DataSync {

class CommandHandler;
class Transport;
class StorageHandler;
class DatabaseHandler;

/**
*/
class ServerCommandHandlerTest : public QObject {
    Q_OBJECT

private slots:
    void initTestCase();
    void cleanupTestCase();

    void testHandlerPut();
    void testHandleGet();
    void testHandleStatus();
    void testResolveConflicts();
    void testAddUIDMapping();
    void testMapUID();
    void testHandleSync();
    void testHandleSyncLargeObject();

private:
    StoragePlugin*        iStorage;
    Transport*            iTransport;
    StorageHandler*       iStorageHandler;
    CommandHandler*       iCommandHandler;
    QString               iRemoteDevice;
    QString               iLocalDb;
    QString               iRemoteDb;
    QList<SyncTarget*>    iTargets;
    ResponseGenerator     iResponseGenerator;
    DatabaseHandler*      iDbHandler;
};

}
#endif
